#!/bin/sh

# Set some visual
if test "x$VISUAL" == "x"; then VISUAL=vi; fi

usage() {
    echo "Usage: $0 <update dir> <tarball> <image id> <version>"
    exit 1
}

if test "$#" != "4"; then usage; fi

UPDATE_DIR=$1
TARBALL=$2
IMAGE=$3
VERSION=$4

if test ! -d $UPDATE_DIR; then echo "Can't find update dir"; exit 1; fi

if test ! -d $UPDATE_DIR/releases; then mkdir $UPDATE_DIR/releases; fi
if test ! -d $UPDATE_DIR/blobs; then mkdir $UPDATE_DIR/blobs; fi

WORKDIR=`mktemp -d /tmp/updatinator.XXXXXXXXXX`
echo "Workdir: $WORKDIR"
mkdir $WORKDIR/image
mkdir $WORKDIR/blobs
mkdir $WORKDIR/releases

tar xjvf $TARBALL -C $WORKDIR/image > /dev/null
if test "$?" != 0; then
    echo "Failed to uncompress tarball"
    echo rm -rf $WORKDIR
    exit 1
fi
generate-manifest -b $WORKDIR/blobs $WORKDIR/image > $WORKDIR/manifest
if test "$?" != 0; then
    echo "Failed to generate manifest"
    echo rm -rf $WORKDIR
    exit 1
fi
MANIFEST_SHA1=`sha1sum $WORKDIR/manifest | head -c 40`
mv $WORKDIR/manifest $WORKDIR/blobs/$MANIFEST_SHA1

# Try to reuse old descr line
DESCR_LINE=`find $UPDATE_DIR/releases -name "$IMAGE.*" | xargs -L 1 grep -h "^descr=" | head -n 1`
FOUND_DESC=1
if test "x$DESCR_LINE" == "x"; then
    DESCR_LINE="descr=<Add description here>"
    FOUND_DESC=0
fi

# Compress blobs
find $WORKDIR/blobs -type f | xargs -L 1 gzip -9


cat > $WORKDIR/release <<EOF
[release]
image=$IMAGE
version=$VERSION
$DESCR_LINE
manifest=$MANIFEST_SHA1
EOF

if test "$FOUND_DESC" == "0"; then
    $VISUAL $WORKDIR/release
fi

mv $WORKDIR/release $UPDATE_DIR/releases/$IMAGE.$VERSION
(cd $WORKDIR/blobs; find . -type f | cpio -p $UPDATE_DIR/blobs)

echo rm -rf $WORKDIR
