/* Build with:
   gcc -o generate-manifest generate-manifest.c sha1.c -Wall -O2
*/

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <dirent.h>
#include <string.h>
#include <stdlib.h>

#include "utils.h"
#include "manifest.h"

static void
collect_blobs (Manifest *m,
	       char *imagedir,
	       char *blobdir)
{
  char *blob_file, *path;
  FileInfo *info;
  int i;

  for (i = 0; i < m->n_infos; i++) {
    info = &m->infos[i];
    if (info->type == FILE_TYPE_BLOB &&
	info->sha1 != NULL &&
	strcmp (info->sha1, "empty") != 0) {
      path = build_path (imagedir, info->name);
      blob_file = build_path (blobdir, info->sha1);
      copy_file (path, blob_file, 0666);
      free (path);
      free (blob_file);
    }
  }
}

static void
usage (void)
{
  fprintf (stderr, "Usage: generate-manifest [-b <blobdir> ] <directory>\n");
}

int
main(int argc, char *argv[])
{
  int arg;
  Manifest *m;
  char *target_blobdir = NULL;
  char *image_dir = NULL;
  struct stat buf;
  
  arg = 1;

  while (arg < argc) {
    if (strcmp (argv[arg], "-b") == 0) {
      arg++;
      if (argc - arg < 1) {
	usage ();
	return 1;
      }
      
      target_blobdir = argv[arg];
    } else {
      break;
    }
    arg++;
  }
  
  if (argc - arg != 1) {
    usage ();
    return 1;
  }

  image_dir = argv[arg];

  if (stat (image_dir, &buf) != 0 || !S_ISDIR (buf.st_mode)) {
    fprintf (stderr, "Image %s is not a directory\n", image_dir);
    return 1;
  }
  
  m = generate_manifest (image_dir);
  if (m == NULL) {
    fprintf (stderr, "Failed to generate manifest\n");
    return 1;
  }

  manifest_print (m, stdout);
  if (target_blobdir)
    collect_blobs (m, argv[arg], target_blobdir);
  
  return 0;
}
