#ifndef MANIFEST_H
#define MANIFEST_H

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>

typedef enum {
  FILE_TYPE_BLOB,
  FILE_TYPE_DIR,
  FILE_TYPE_CHAR,
  FILE_TYPE_BLOCK,
  FILE_TYPE_FIFO,
  FILE_TYPE_LINK,
  FILE_TYPE_SOCKET
} FileType;

typedef struct {
  char *name;
  FileType type;
  mode_t mode;
  uid_t uid;
  gid_t gid;
  dev_t dev;
  char *symlink;
  char *sha1;
} FileInfo;

typedef struct {
  FileInfo *infos;
  int n_infos;
  int n_allocated;
} Manifest;

typedef int (*file_info1_callback) (FileInfo *info,
				    void *data);
typedef int (*file_info2_callback) (FileInfo *info1,
				    FileInfo *info2,
				    void *data);

Manifest *manifest_new      (void);
Manifest *generate_manifest (char                *path);
void      manifest_free     (Manifest            *manifest);
void      manifest_add_file (Manifest            *manifest,
			     FileInfo            *info);
void      manifest_sort     (Manifest            *manifest);
Manifest *parse_manifest    (char                *path);
int       manifest_diff     (Manifest            *from,
			     Manifest            *to,
			     int                  reverse,
			     file_info1_callback  added,
			     file_info1_callback  removed,
			     file_info2_callback  changed,
			     void                *data);
void      manifest_print    (Manifest            *manifest,
			     FILE                *out);


FileType get_file_type_from_stat (struct stat *buf);
int      file_info_equal         (FileInfo    *info1,
				  FileInfo    *info2);
void     destroy_file_info       (FileInfo    *info);
void     print_file_info         (FileInfo    *info,
				  FILE        *out);
int      file_info_create        (FileInfo    *info,
				  const char  *image_dir,
				  const char  *blob_dir,
				  int          blob_fd);

#endif /* MANIFEST_H */
